require 'minitest/autorun'
require 'minitest/reporters'
require 'pepito/adapters/slack'

Minitest::Reporters.use!(
  Minitest::Reporters::SpecReporter.new,
  ENV,
  Minitest.backtrace_filter
)

class StubSlack < Pepito::Adapters::Slack
  attr_accessor :client
end

class TestPepitoAdaptersSlack < MiniTest::Test
  def test_is_module
    assert_kind_of Module, Pepito::Adapters::Slack
  end

  def test_fail_auth_token
    assert_raises Slack::Web::Api::Error do
      create_slack(nil, '1234567890')
    end
  end

  def test_auth
    create_slack(nil, ENV['SLACK_API_TOKEN'])
    pass
  end

  def test_run
    mock_client = MiniTest::Mock.new
    mock_client.expect(:start!, nil)

    slack = create_stub_slack(nil, ENV['SLACK_API_TOKEN'])
    slack.client = mock_client
    slack.run

    assert mock_client.verify
  end

  def test_send_messages
    room = 'room'
    message = "1\n2\n3\n"

    mock_target = MiniTest::Mock.new
    mock_target.expect(:room, room)

    mock_client = MiniTest::Mock.new
    mock_client.expect(:message, nil, [channel: room, text: message])

    slack = create_stub_slack(nil, ENV['SLACK_API_TOKEN'])
    slack.client = mock_client
    slack.send_messages(mock_target, message.split)

    assert mock_target.verify
    assert mock_client.verify
  end

  def test_topic
    room = 'room'
    topic = 'topic'

    mock_target = MiniTest::Mock.new
    mock_target.expect(:room, room)

    mock_web_client = MiniTest::Mock.new
    mock_web_client.expect(:channels_setTopic, nil, [channel: room, topic: topic])

    mock_client = MiniTest::Mock.new
    mock_client.expect(:web_client, mock_web_client)

    slack = create_stub_slack(nil, ENV['SLACK_API_TOKEN'])
    slack.client = mock_client
    slack.topic(mock_target, topic)

    assert mock_target.verify
    assert mock_web_client.verify
    assert mock_client.verify
  end

  def test_stop
    mock_client = MiniTest::Mock.new
    mock_client.expect(:stop!, nil)

    slack = create_stub_slack(nil, ENV['SLACK_API_TOKEN'])
    slack.client = mock_client
    slack.stop

    assert mock_client.verify
  end

  private

  def create_slack(mock_robot, token)
    Pepito::Adapters::Slack.new(mock_robot, 'SLACK_API_TOKEN' => token)
  end

  def create_stub_slack(mock_robot, token)
    StubSlack.new(mock_robot, 'SLACK_API_TOKEN' => token)
  end
end
